import 'dart:async';
import 'package:flutter/services.dart';
export './api/entity.dart' show fetchEntity;
export './api/file.dart' show fetchFileString, fetchFileBytes;
export './models/entity.dart'
    show Entity, EntityAction, EntityActionBrowser, EntityActionImage;
export './util/content-uri.dart' show ContentURI;

final defaultHdPath = "m/44'/60'/0'/0/0";

class Dvote {
  static const MethodChannel _channel = const MethodChannel('dvote');

  // //////////////////////////////////////////////////////////////////////////
  // WALLET MANAGEMENT
  // //////////////////////////////////////////////////////////////////////////

  /// Generate a random private key and encode it with a human readable BIP39 mnemonic.
  /// Entropy is the bit count (128, ..., 256)
  static Future<String> generateMnemonic({int size = 192}) async {
    final String mnemonic =
        await _channel.invokeMethod<String>('generateMnemonic', size);
    return mnemonic;
  }

  /// Return the private key that corresponds to the given mnemonic and (optional) HD derivation path
  static Future<String> mnemonicToPrivateKey(String mnemonic,
      {String hdPath}) async {
    if (hdPath == null || hdPath == "") hdPath = defaultHdPath;
    final String privKey = await _channel
        .invokeMethod<String>('mnemonicToPrivateKey', [mnemonic, hdPath]);
    return privKey;
  }

  /// Return the public key that corresponds to the given mnemonic and (optional) HD derivation path
  static Future<String> mnemonicToPublicKey(String mnemonic,
      {String hdPath}) async {
    if (hdPath == null || hdPath == "") hdPath = defaultHdPath;
    final String pubKey = await _channel
        .invokeMethod<String>('mnemonicToPublicKey', [mnemonic, hdPath]);
    return pubKey;
  }

  /// Return the Ethereum address that corresponds to the given mnemonic and (optional) HD derivation path
  static Future<String> mnemonicToAddress(String mnemonic,
      {String hdPath}) async {
    if (hdPath == null || hdPath == "") hdPath = defaultHdPath;
    final String addr = await _channel
        .invokeMethod<String>('mnemonicToAddress', [mnemonic, hdPath]);
    return addr;
  }

  // //////////////////////////////////////////////////////////////////////////
  // SIGNATURE
  // //////////////////////////////////////////////////////////////////////////

  /// Sign the given payload using the private key and return a hex signature
  static Future<String> signString(String payload, String privateKey) async {
    if (payload == null)
      throw "The payload is empty";
    else if (privateKey == null) throw "The privateKey is empty";

    final String result = await _channel
        .invokeMethod<String>('signString', [payload, privateKey]);
    return result;
  }

  /// Check whether the given signature is valid and belongs to the given message and
  /// public key
  static Future<bool> verifySignature(
      String hexSignature, String strPayload, String hexPublicKey) async {
    if (hexSignature == null)
      throw "The hexSignature is empty";
    else if (strPayload == null)
      throw "The payload is empty";
    else if (hexPublicKey == null) throw "The hexPublicKey is empty";

    final bool result = await _channel.invokeMethod<bool>(
        'verifySignature', [hexSignature, strPayload, hexPublicKey]);
    return result;
  }

  // //////////////////////////////////////////////////////////////////////////
  // LOCAL ENCRYPTION
  // //////////////////////////////////////////////////////////////////////////

  /// Encrypt the given payload using passphrase and return it as a string
  static Future<String> encryptString(String payload, String passphrase) async {
    if (payload == null)
      throw "The payload is empty";
    else if (passphrase == null) throw "The passphrase is empty";

    final String result = await _channel
        .invokeMethod<String>('encryptString', [payload, passphrase]);
    return result;
  }

  /// Decrypt the given payload using passphrase and return the original text
  static Future<String> decryptString(
      String encryptedPayload, String passphrase) async {
    if (encryptedPayload == null)
      throw "The encryptedPayload is empty";
    else if (passphrase == null) throw "The passphrase is empty";

    final String result = await _channel
        .invokeMethod<String>('decryptString', [encryptedPayload, passphrase]);
    return result;
  }
}
